<?php

declare(strict_types=1);

namespace Drupal\mailer_policy\Plugin\EmailAdjuster;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\mailer_policy\Attribute\EmailAdjuster;
use Drupal\mailer_policy\EmailAdjusterBase;
use Drupal\symfony_mailer\EmailInterface;

/**
 * Defines the Body Email Adjuster.
 */
#[EmailAdjuster(
  id: "email_body",
  label: new TranslatableMarkup("Body"),
  description: new TranslatableMarkup("Sets the email body."),
)]
class BodyEmailAdjuster extends EmailAdjusterBase implements TrustedCallbackInterface {

  /**
   * {@inheritdoc}
   */
  public function build(EmailInterface $email): void {
    $content = $this->configuration['content'];

    $body = [
      '#type' => 'processed_text',
      '#text' => $content['value'],
      '#format' => $content['format'] ?? filter_default_format(),
    ];

    $variables = $email->getVariables();
    if ($existing_body = $email->getBody()) {
      $variables['body'] = $existing_body;
    }

    if ($variables) {
      $body += [
        '#pre_render' => [
          // Preserve the default pre_render from the element.
          [$this, 'preRenderVariables'],
          ['Drupal\filter\Element\ProcessedText', 'preRenderText'],
        ],
        '#context' => $variables,
      ];
    }

    $email->setBody($body);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $form['content'] = [
      '#title' => $this->t('Content'),
      '#type' => 'text_format',
      '#default_value' => $this->configuration['content']['value'] ?? NULL,
      '#format' => $this->configuration['content']['format'] ?? filter_default_format(),
      '#required' => TRUE,
      '#rows' => 10,
      '#description' => $this->t('Email body.') . $form_state->getValue('mailer_policy_help'),
    ];

    return $form;
  }

  /**
   * Pre-render callback for replacing twig variables.
   */
  public function preRenderVariables(array $body): array {
    $twig_service = \Drupal::service('twig');
    $body['#text'] = $twig_service->renderInline($body['#text'], $body['#context']);
    return $body;
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks(): array {
    return ['preRenderVariables'];
  }

}
