<?php

declare(strict_types=1);

namespace Drupal\symfony_mailer\Processor;

use Drupal\Component\Render\PlainTextOutput;
use Drupal\Component\Utility\Html;
use Drupal\Core\Utility\Token;
use Drupal\symfony_mailer\EmailInterface;

/**
 * Email processor for token replacement.
 */
class TokenProcessor implements EmailProcessorInterface {

  use EmailProcessorTrait;

  /**
   * The data for token replacement.
   */
  protected array $data;

  /**
   * The options for token replacement.
   */
  protected array $options = [];

  /**
   * The token-replacement service.
   */
  protected Token $token;

  /**
   * Whether tokens have been replaced in the body.
   */
  protected bool $replacedBody = FALSE;

  /**
   * {@inheritdoc}
   */
  public function build(EmailInterface $email): void {
    /** @var */
    $this->token = \Drupal::token();
    $this->data ??= $email->getParams();

    if ($subject = $email->getSubject()) {
      $subject = PlainTextOutput::renderFromHtml($this->token->replace(Html::escape($subject), $this->data, $this->options));
      $email->setSubject($subject);
    }

    $body = $email->getBody();
    if ($body && isset($body['#type']) && ($body['#type'] == 'processed_text')) {
      $body['#text'] = $this->token->replace($body['#text'], $this->data, $this->options);
      $email->setBody($body);
      $this->replacedBody = TRUE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function postRender(EmailInterface $email): void {
    if (!$this->replacedBody && $body = $email->getHtmlBody()) {
      $email->setHtmlBody($this->token->replace($body, $this->data, $this->options));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getWeight(int $phase): int {
    if ($phase == EmailInterface::PHASE_BUILD) {
      return 900;
    }
    return EmailInterface::DEFAULT_WEIGHT;
  }

  /**
   * Sets data for token replacement.
   *
   * @param array $data
   *   An array of keyed objects.
   *
   * @return $this
   */
  public function setData(array $data): static {
    $this->data = $data;
    return $this;
  }

  /**
   * Sets options for token replacement.
   *
   * @param array $options
   *   A keyed array of settings and flags.
   *
   * @return $this
   */
  public function setOptions(array $options): static {
    $this->options = $options;
    return $this;
  }

}
